-- Warborg is Wargames playing on network
-- Copyright (C) 2011 Manuel De Girardi

-- This Program is Free Software: You Can Redistribute It and/or Modify
-- It Under The Terms of The GNU General Public License As Published By
-- The Free Software Foundation, Either Version 3 of The License, or
-- (at Your Option) Any Later Version.

-- This Program is Distributed in The Hope That It Will Be Useful,
-- But WITHOUT ANY WARRANTY; Without Even The Implied Warranty of
-- MERCHANTABILITY or FITNESS for A PARTICULAR PURPOSE.  See The
-- GNU General Public License for More Details.

-- You Should Have Received A Copy of The GNU General Public License
-- Along with This Program.  if not, See <Http://Www.Gnu.Org/Licenses/>.

with Text_Io;
use Text_Io;

with Ada.Command_Line;
use Ada.Command_Line;

with Warborg.Common, Warborg.Database;
use Warborg.Common, Warborg.Database;

with Ada.Strings.Fixed;
use Ada.Strings;

with Ada.Streams.Stream_Io;
use Ada.Streams;

with ada.Task_Identification;
use ada.Task_Identification;

with Gnat.MD5, Gnat.Sockets;
use Gnat;

with Ada.Unchecked_Deallocation;

package body Warborg.Uniborg is

   
   procedure Usage is
      
   begin
      Text_Io.Put_Line("usage : " & Command_Name & " Hostname");
   end Usage;


   type String_Access is access String;
   
   package Users is
      function Add_User(Logname : in String;
                        Passwd : in String) return Boolean;

      function Check_passwd(Logname : in String;
                            Passwd : in String) return Boolean;
   end Users;

   package body Users is



      subtype T_Logname is String(1..64);
      subtype T_Passwd is String(1..32);

      type T_User is
         record
            Logname : T_Logname := (others => character'val(32));
            Passwd : T_Passwd := (others => character'val(32));
         end record;

      Users_Filename : constant String := "users.bin";

      function Add_User(Logname : in String;
                        Passwd : in String) return Boolean is

         Users_File : Stream_Io.File_Type;
         Users_Stream : Stream_Io.Stream_Access;

         Old, Current : T_User;
         Success : Boolean := False;
      begin

         Current.Logname(1..Logname'Length) := Logname;
         Current.Passwd(1..Passwd'Length) := Passwd;
         begin

            Stream_IO.open(users_File, Stream_Io.In_File, users_Filename);

            users_stream := Stream_IO.Stream(users_File);
            while not Stream_Io.End_Of_File(Users_File) loop
               T_User'read(Users_Stream, old);
               if Current.Logname = Old.Logname then
                  Stream_IO.Close(Users_File);
                  return False;
               end if;
            end loop;
            Stream_IO.Close(Users_File);
         exception
            when Stream_Io.Name_Error =>
               Stream_IO.create(users_File, Stream_Io.out_File, users_Filename);
               Stream_IO.Close(Users_File);
         end;

         Stream_IO.open(users_File, Stream_Io.append_File, users_Filename);
         users_stream := Stream_IO.Stream(users_File);
         T_User'write(Users_Stream, Current);
         Stream_IO.Close(Users_File);
         return True;

      exception

         when others =>
            return False;
      end add_user;


      function Check_passwd(Logname : in String;
                            Passwd : in String) return Boolean is
         Users_File : Stream_Io.File_Type;
         Users_Stream : Stream_Io.Stream_Access;


         Old, Current : T_User;

      begin

         Current.Logname(1..Logname'Length) := Logname;
         Current.Passwd(1..Passwd'Length) := Passwd;

         Stream_IO.open(users_File, Stream_Io.In_File, users_Filename);

         users_stream := Stream_IO.Stream(users_File);

         while not Stream_Io.End_Of_File(Users_file) loop
            T_User'read(Users_Stream, old);

            if Current.Logname = Old.Logname then

               if Current.passwd = Old.passwd then

                  Stream_IO.Close(Users_File);

                  return True;

               else

                  Stream_IO.Close(Users_File);
                  return False;
               end if;
            end if;
         end loop;

         Stream_IO.Close(Users_File);
         return False;
      exception
         when others =>
            return False;
      end Check_passwd;
   end Users;




   Sgbd : Sgbd_Type;







   type Socket_Access is access all Sockets.Socket_Type;


   task type T_Thread_Warborg(Socket : Socket_Access) is
      entry Get_Id(Id : out Task_Id);
   end T_Thread_Warborg;

   type Thread_Warborg_Access is access T_Thread_Warborg;

   type T_Thread_Warborg_Task is
      record
         Id : Task_Id := Null_Task_id;
         thread_Warborg : thread_Warborg_access;
      end record;
   type thread_Warborg_Task_Access is access T_Thread_Warborg_Task;



   procedure Unchecked_Deallocation_thread_Warborg is
      new Ada.Unchecked_Deallocation(T_Thread_Warborg,thread_Warborg_Access);
   procedure Unchecked_Deallocation_thread_Warborg_task is
      new Ada.Unchecked_Deallocation(T_Thread_Warborg_Task,thread_Warborg_Task_Access);

   task body T_Thread_Warborg is





      Channel  : constant Sockets.Stream_Access := Sockets.stream(socket.all);

      Output, United_states, Soviet_Union, Clear_Passwd, Input, Logname, Passwd : String_access;

      Games_List : String_Access := new String ' (
                                                  "Falken'S Maze" & Character'Val(10) &
                                                    "Black Jack" & Character'Val(10) &
                                                    "Gin Rummy" & Character'Val(10) &
                                                    "Hearts" & Character'Val(10) &
                                                    "Bridge" & Character'Val(10) &
                                                    "Checkers" & Character'Val(10) &
                                                    "Chess" & Character'Val(10) &
                                                    "Poker" & Character'Val(10) &
                                                    "Fighter Combat" & Character'Val(10) &
                                                    "Guerrila Engagement" & Character'Val(10) &
                                                    "Desert Warfare" & Character'Val(10) &
                                                    "Air-To-Ground Actions" & Character'Val(10) &
                                                    "Theaterwide Tactical Warfare" & Character'Val(10) &
                                                    "Theaterwide Biotoxic and Chemical Warfare" & Character'Val(10) &
                                                    Character'Val(10) &
                                                    "Global Thermonuclear War"
                                                 );
      Command : Command_Type;
      Term_Address      : Sockets.Sock_Addr_Type := Sockets.Get_Peer_Name(Socket.all);
      
      
      Side : Side_Type;
      Success : Boolean := False;
      Wopr : Wopr_Type;
   begin

      accept get_id(id : out task_id) do
         id := current_task;
      end get_id;
      
      Output := new String ' (Character'Val(10) &
				"    United States                  Soviet Union" & Character'Val(10) &
				"     /----------\                  /-----------\" & Character'Val(10) &
				"    /            \                /             \" & Character'Val(10) &
				"   /              \              /               \" & Character'Val(10) &
				"  /                \            /                 \" & Character'Val(10) &			
				" |                 |           |               ____|" & Character'Val(10) &
				" |                 |           |              /" & Character'Val(10) &
				" \_________________/           \_____________/" & Character'Val(10) 
				& Character'Val(10) &
				Character'Val(10));
      
      
      
      
      
      
      United_states := new String ' (Character'Val(10) &
				       "    United States                  Soviet Union" & Character'Val(10) &
				       "     /----------\                  /-----------\" & Character'Val(10) &
				       "    /            \                /             \" & Character'Val(10) &
				       "   /              \              /               \" & Character'Val(10) &
				       "  /                \            /                 \" & Character'Val(10) &       
				       " |                 |           |               ____|" & Character'Val(10) &
				       " |                 |           |              /" & Character'Val(10) &
				       " \_________________/           \_____________/"	& Character'Val(10) &
				       "         You                          Joshua"  & Character'Val(10) &	       
				       Character'Val(10));
      
      Soviet_union := new String ' (Character'Val(10) &
				      "    United States                  Soviet Union" & Character'Val(10) &
				      "     /----------\                  /-----------\" & Character'Val(10) &
				      "    /            \                /             \" & Character'Val(10) &
				      "   /              \              /               \" & Character'Val(10) &
				      "  /                \            /                 \" & Character'Val(10) &
				      " |                 |           |               ____|" & Character'Val(10) &
				      " |                 |           |              /" & Character'Val(10) &
				      " \_________________/           \_____________/"  & Character'Val(10) &
				      "       Joshua                           You"  & Character'Val(10) &	      
				      Character'Val(10));

      
      
      String'Output(Channel, Character'Val(10) & "LOGON: ");
      loop
         Input := new String ' (String'Input(Channel));
         if Input.all = "help games" then
            Boolean'Output(Channel, false);
            String'Output(Channel,
                          Games_list.all & Character'Val(10) & "LOGON: ");
         elsif Input.all = "help" then
            Boolean'Output(Channel,
                           False);
            String'Output(Channel,
                          "help not available." & Character'Val(10) & "LOGON: ");

         elsif Logname = null and Input'Length /= 0 then
            Boolean'Output(Channel, false);
            Logname := new String ' (Input.all);
            String'Output(Channel, Character'Val(10) &"PASSWORD: ");
         elsif Logname /= null then
            Clear_Passwd := new String ' (Input.all);
            Passwd := new String ' (Gnat.Md5.Digest(Clear_Passwd.all));
            if Users.Add_User(Logname.all, Passwd.all) or
              Users.Check_Passwd(Logname.all, Passwd.all) then
               Boolean'Output(Channel, True);	       	       
	       String'Output(Channel, Logname.all);	       
	       Sgbd.Get_Wopr(Logname.all, Wopr, Success);	       
	       Text_Io.Put("Envoie wopr exist");
	       Boolean'Output(Channel, success);
	       if not Success then
		  String'Output(Channel, Character'Val(10) & "LOGON: ");
		  Text_Io.Put("Joshua -1");
		  loop
		     input := new String ' (String'Input(Channel));	 
		     Text_Io.Put("Joshua 0");
		     if Input.all = "help games" then
			Boolean'Output(Channel, false);
			String'Output(Channel,
				      Games_list.all & Character'Val(10) & "LOGON: ");
		     elsif Input.all = "help" then
			Boolean'Output(Channel, False);
			String'Output(Channel,
				      "help not available." & Character'Val(10) & "LOGON: ");
			
		     elsif  Logname /= null and then Input /= null and then Input'Length /= 0 then
			if Input.all = "Joshua" then
			   Text_Io.Put("Joshua 1");
			   Boolean'Output(Channel, False);
			   String'Output(Channel,
					 "Greetings, Professor Falken." & Character'Val(10) &
					   Character'Val(10) &
					   "Why we do have deleted your account Avril 22, 1970?" & Character'Val(10));
			   --
			   loop
			      input := new String ' (String'Input(Channel));	 
			      -- if "orry" do not exist in input.
			      if not (0 /= Fixed.index(Input.all, "orry")) then
				 Boolean'Output(Channel, false);
				 String'Output(Channel,
					       Character'Val(10) &
						 "Sorry! " & Character'Val(10));
				 
			      else
				 
				 
				 Boolean'Output(Channel, false);
				 String'Output(Channel,
					       Character'Val(10) &
						 "Shall we play a game? " & Character'Val(10));
				 exit;
				 
			      end if;
			   end loop;
			   
			   Text_Io.Put("Joshua 2");
			   loop
			      input := new String ' (String'Input(Channel));	 
			      
			      if Input.all = "Global Thermonuclear War" then
				 Boolean'Output(Channel, False);
				 String'Output(Channel, "Wouldn't you prefer a good game of chess? " & Character'Val(10));
				 Input := new String ' (String'Input(Channel));
				 
				 if Fixed.Index(Input.all, "Global Thermonuclear War") /= 0 then				 
				    
				    --
				    loop		
				       --
				       Boolean'Output(Channel, false);
				       String'Output(Channel, "Excellent..." & 
						       Character'Val(10) &
						       Output.all &
						       "WHICH SIDE DO YOU WANT?" & Character'Val(10)
						       & Character'Val(10) &
						       Character'Val(10) &					
						       "1. UNITED STATES" & Character'Val(10) &
						       "2. SOVIET UNION" & Character'Val(10)
						       & Character'Val(10) &
						       Character'Val(10) &
						       
						       "PLEASE CHOOSE ONE: ");
				       --
				       
				       input := new String ' (String'Input(Channel));	 
				       
				       if Input.all = "UNITED STATES" or Input.all = "1" then

					  Output := new String ' (United_States.all);
					  Side := Ouest;
					  exit;
				       elsif Input.all = "SOVIET UNION" or Input.all = "2" then
					  Side := est;
					  Output := new String ' (Soviet_Union.all);
					  exit;
				       end if;
				       --
				    end loop;
				    
				    
				    
				    Boolean'Output(Channel, False);
				    String'Output(Channel, "                      AWAITING FIRST STRIKE COMMAND" & Character'Val(10) &
						    "================================================================================" &
						    Output.all
						    & Character'Val(10) &
						    Character'Val(10) &						   
						    "PLEASE LIST PRIMARY TARGETS BY" & Character'Val(10) &
						    "CITY AND/OR COUNTY NAME:" & Character'Val(10));
				    
				    
				    
				    
				    input := new String ' (String'Input(Channel));				    
				    Boolean'Output(Channel, True);				 				 
				    Text_Io.Put("Envoy good luck 1");
				    String'Output(Channel, "Good luck");				 
				    Text_Io.Put("Envoy side");
				    Side_Type'Output(Channel, Side);
				    
				    
				    begin
				       
				       
				       loop
					  Command := Command_Type'Input(Channel);
					  case Command is
					     when Wopr_Create =>
						declare
						   Wopr : Wopr_Type;
						   Success : Boolean;
						begin
						   
						   
						   
						   
						   Wopr.Wopr_Attributes := Wopr_Attributes_Type'Input(Channel);

						   Sgbd.Add_Wopr(Logname.all, Wopr, success);

						   Boolean'Output(Channel, Success);
						   if Success then
						      Put_Line("New wopr");
						   else
						      Put_Line("Bad comportment.");
						   end if;
						end;
					     when Wopr_get =>
						declare
						   Wopr : Wopr_Type;
						   Success : Boolean;
						begin

						   Sgbd.Get_Wopr(Logname.all, Wopr, Success);

						   Boolean'Output(Channel, success);
						   if Success then

						      Wopr_Attributes_Type'Output(Channel, Wopr.Wopr_attributes);
						      if not Missiles_Vectors.Is_Empty(Wopr.Missiles) then
							 Boolean'Output(Channel, True);
							 Missile_Vector_Index_Type'Output(Channel, Missiles_Vectors.Last_Index(Wopr.Missiles));

							 for I in 1..Missiles_Vectors.Last_Index(Wopr.Missiles) loop

							    Missile_Attributes_Type'output(Channel,
											   Missiles_Vectors.Element(Wopr.Missiles, I).Missile_attributes);
							 end loop;

						      else
							 Boolean'Output(Channel, False);


						      end if;
						   end if;
						end;
					     when Missile_Create =>
						declare
						   Missile_Attributes : Missile_Attributes_Type;
						   Missile : Missile_Type;
						   Success : Boolean := False;
						begin

						   Missile_Attributes := Missile_Attributes_Type'Input(Channel);

						   Missile.Missile_Attributes := Missile_Attributes;

						   Sgbd.Add_Missile(Logname.all, Missile, success);

						   Boolean'Output(Channel, success);
						end;
					     when Launch_One =>
						declare
						   Wopr : Wopr_Type;
						   One_Missile_Launched : Missile_Vector_Index_Type;
						   Success : Boolean := False;
						begin
						   One_Missile_Launched := Missile_Vector_Index_type'Input(Channel);
						   Sgbd.Launch_One(Logname.all, One_Missile_Launched);
						   Sgbd.Get_Wopr(Logname.all, Wopr, Success);
						   Boolean'Output(Channel, success);
						end;
					     when Launch_All =>
						declare
						   Wopr : Wopr_Type;
						   Success : Boolean := False;
						begin
						   Sgbd.Launch_all(Logname.all);
						   Sgbd.Get_Wopr(Logname.all, Wopr, Success);
						   Boolean'Output(Channel, success);
						end;
					     when Enemy_Get =>
						declare
						   Enemy : Missiles_Vectors.Vector;
						begin
						   Sgbd.Get_Enemy(Logname.all, enemy);
						   if not Missiles_Vectors.Is_Empty(Enemy) then
						      Boolean'Output(Channel, True);
						      Missile_Vector_Index_Type'Output(Channel, Missiles_Vectors.Last_Index(Enemy));

						      for I in 1..Missiles_Vectors.Last_Index(Enemy) loop

							 Missile_Attributes_Type'output(Channel,
											Missiles_Vectors.Element(Enemy, I).Missile_attributes);
						      end loop;

						   else
						      Boolean'Output(Channel, False);
						   end if;
						end;
					     when others =>
						null;
					  end case;
				       end loop;
				    end;
				    
				    
				 else
				    
				    Boolean'Output(Channel, False);
				    String'Output(Channel, "Fine." & Character'Val(10) & "LOGON: ");
				    exit;
				    
				 end if;
				 
				 
			      else
				 
				 Boolean'Output(Channel, false);
				 String'Output(Channel, "Game not available." & Character'Val(10) & "LOGON: ");
				 exit;
			      end if;
			      
			   end loop;

			   
			   



			   --

			else
			   Boolean'Output(Channel, false);
			   String'Output(Channel, "Game not available." & Character'Val(10) & "LOGON: ");
			end if;
		     else
			Text_Io.Put("Joshua 6");
			Boolean'Output(Channel, false);
			String'Output(Channel,
				      "Access denied." & Character'Val(10) & "LOGON: ");
			Logname := null;
		     end if;	--
		     
		  end loop;	  
		  
	       else
		  Text_Io.Put("Envoy good luck 2");
		  String'Output(Channel, "Good luck");				 
		  Side_Type'Output(Channel, Wopr.Wopr_Attributes.Side);
		  
		  
		  
		  loop
		     
		     begin
			
			Command := Command_Type'Input(Channel);
			case Command is
			   when Wopr_Create =>
			      declare
				 Wopr : Wopr_Type;
				 Success : Boolean;
			      begin
				 
				 
				 
				 
				 Wopr.Wopr_Attributes := Wopr_Attributes_Type'Input(Channel);

				 Sgbd.Add_Wopr(Logname.all, Wopr, success);

				 Boolean'Output(Channel, Success);
				 if Success then
				    Put_Line("New wopr");
				 else
				    Put_Line("Bad comportment.");
				 end if;
			      end;
			   when Wopr_get =>
			      declare
				 Wopr : Wopr_Type;
				 Success : Boolean;
			      begin

				 Sgbd.Get_Wopr(Logname.all, Wopr, Success);

				 Boolean'Output(Channel, success);
				 if Success then

				    Wopr_Attributes_Type'Output(Channel, Wopr.Wopr_attributes);
				    if not Missiles_Vectors.Is_Empty(Wopr.Missiles) then
				       Boolean'Output(Channel, True);
				       Missile_Vector_Index_Type'Output(Channel, Missiles_Vectors.Last_Index(Wopr.Missiles));

				       for I in 1..Missiles_Vectors.Last_Index(Wopr.Missiles) loop

					  Missile_Attributes_Type'output(Channel,
									 Missiles_Vectors.Element(Wopr.Missiles, I).Missile_attributes);
				       end loop;

				    else
				       Boolean'Output(Channel, False);


				    end if;
				 end if;
			      end;
			   when Missile_Create =>
			      declare
				 Missile_Attributes : Missile_Attributes_Type;
				 Missile : Missile_Type;
				 Success : Boolean := False;
			      begin

				 Missile_Attributes := Missile_Attributes_Type'Input(Channel);

				 Missile.Missile_Attributes := Missile_Attributes;

				 Sgbd.Add_Missile(Logname.all, Missile, success);

				 Boolean'Output(Channel, success);
			      end;
			   when Launch_One =>
			      declare
				 Wopr : Wopr_Type;
				 One_Missile_Launched : Missile_Vector_Index_Type;
				 Success : Boolean := False;
			      begin
				 One_Missile_Launched := Missile_Vector_Index_type'Input(Channel);
				 Sgbd.Launch_One(Logname.all, One_Missile_Launched);
				 Sgbd.Get_Wopr(Logname.all, Wopr, Success);
				 Boolean'Output(Channel, success);
			      end;
			   when Launch_All =>
			      declare
				 Wopr : Wopr_Type;
				 Success : Boolean := False;
			      begin
				 Sgbd.Launch_all(Logname.all);
				 Sgbd.Get_Wopr(Logname.all, Wopr, Success);
				 Boolean'Output(Channel, success);
			      end;
			   when Enemy_Get =>
			      declare
				 Enemy : Missiles_Vectors.Vector;
			      begin
				 Sgbd.Get_Enemy(Logname.all, enemy);
				 if not Missiles_Vectors.Is_Empty(Enemy) then
				    Boolean'Output(Channel, True);
				    Missile_Vector_Index_Type'Output(Channel, Missiles_Vectors.Last_Index(Enemy));

				    for I in 1..Missiles_Vectors.Last_Index(Enemy) loop

				       Missile_Attributes_Type'output(Channel,
								      Missiles_Vectors.Element(Enemy, I).Missile_attributes);
				    end loop;

				 else
				    Boolean'Output(Channel, False);
				 end if;
			      end;
			   when others =>
			      null;
			end case;
			
		     end;
		     
		  end loop;
		  
		  
	       end if;
	       
	    else
	       Boolean'Output(Channel, false);
	       String'Output(Channel,
			     "Access denied." & Character'Val(10) & "LOGON: ");
	       
	       Logname := null;   	    	    
	    end if;
	 end if;	 	 
      end loop;

   exception
      when others =>
         declare
            Wopr : Wopr_Type;
            Input : String_Access;
            Socket            : Sockets.Socket_Type;
            Channel           : Sockets.Stream_access;
            Success : Boolean;
         begin
            Put("Call reborg...");
            delay 5.0;
            Sgbd.Get_Wopr(Logname.all, Wopr, Success);
	    if Success then
	       declare
		  task Client_Recall is
		  end Client_Recall;
		  task body Client_Recall is
		  begin
		     Term_Address.Port := Reborg_Port;
		     Sockets.Create_Socket (Socket);
		     
		     Sockets.Set_Socket_Option
		       (Socket,
			Sockets.Socket_Level,
			(Sockets.Reuse_Address, True));
		     
		     delay 0.2;
		     
		     Sockets.Connect_Socket (Socket, Term_Address);
		     Channel :=  Sockets.stream(Socket);                  
		  
		     Boolean'Output(Channel, True);
		     Put_Line("Success ");
		     String'Output(Channel, Argument(1));
		     String'Output(Channel, Logname.all);
		     String'Output(Channel, Clear_Passwd.all);
		     delay 2.0;
		     Close_Socket(Socket);
		  end Client_Recall;
		  
	       begin
		  delay 10.0;
		  abort Client_Recall;
	       end;
	    end if;
            
            Put_Line(" Done");
         end;
   end T_Thread_Warborg;

   
   
   task body Warborg_Server is
      
      Max : constant Positive := 250;
      
      Liste : array(1..max) of Socket_access;
      Liste_Thread_task : array(1..max) of Thread_Warborg_Task_Access;
      Bliste : array(1..max) of Boolean;
      Reads,Write : Sockets.Socket_Set_Type;

      Selector : Sockets.Selector_Type;
      Srv_Adr,Adr : Sockets.Sock_Addr_Type;
      Srv_Socket : Sockets.Socket_Type;
      Status : Sockets.Selector_Status;

      Ptr : Integer := 0;

      function Chercher return Natural is
      begin
         for I in 1..max loop
            if Liste_Thread_Task(I) = null then
               return I;            
            elsif Is_Terminated(Liste_Thread_Task(I).Id) then
               Unchecked_Deallocation_Thread_Warborg(Liste_Thread_Task(I).Thread_Warborg);
               Unchecked_Deallocation_Thread_Warborg_task(Liste_Thread_Task(I));
               Sockets.Close_Socket(Liste(I).all);
               return I;
            end if;
         end loop;
         return 0;
      end Chercher;

      thread_Suivant : Natural := 0;

      End_Of_Task : Boolean := False;
      Run : Boolean := False;
   begin
      
      if Argument_Count /= 1 then
	 Usage;
	 raise Program_Error;
      end if;
      
      Sockets.Initialize;

      for I in 1..max loop
         liste(I) := new Sockets.Socket_Type;
      end loop;


      Bliste := (others => False);

      Srv_Adr.Addr := Sockets.Any_Inet_Addr;
      Srv_Adr.Port := 5872;
      Sockets.Create_Socket(Srv_Socket);

      Sockets.Bind_Socket(Srv_Socket,Srv_Adr);
      Sockets.Listen_Socket(Srv_Socket);
      Sockets.Create_Selector(Selector);
      Sockets.Set(Write,Srv_Socket);

      
      
      loop
	 begin
	    Sockets.Empty(Reads);
	    
	    Sockets.Set(Reads,Srv_Socket);
	    
	    for I in 1..max loop
	       if Bliste(I) then		  
		  Sockets.Set(Write,Liste(I).all);
	       end if;
	    end loop;
	    
	    Sockets.Check_Selector(Selector, Reads, Write, Status);

	    case Status is
	       when Sockets.Completed =>

		  if Sockets.Is_Set(Reads, Srv_Socket) or Sockets.Is_Set(Write, Srv_Socket)  then
		     Thread_Suivant := Chercher;

		     if Thread_Suivant /= 0 then
			Ptr := Thread_Suivant;

			Sockets.Accept_Socket(Srv_Socket,Liste(Ptr).all,Adr);
			Bliste(Ptr) := True;
			
		     end if;
		  else
		     
		     for I in 1..max loop


			if bliste(I) then

			   if Sockets.Is_Set(Reads,Liste(I).all) or Sockets.Is_Set(Write,Liste(I).all) then
			      Liste_Thread_Task(I) := new T_Thread_Warborg_Task;
			      
			      Liste_Thread_Task(I).Thread_Warborg := new T_Thread_Warborg(Liste(i));

			      Liste_Thread_Task(i).Thread_Warborg.Get_Id(Liste_Thread_Task(i).Id);
			      

			      Bliste(i) := false;

			   end if;
			end if;
		     end loop;
		  end if;
		  delay 1.0;
	       when Sockets.Expired =>
		  Text_Io.Put_Line("expired");
	       when Sockets.Aborted =>
		  Text_Io.Put_Line("aborted");
	    end case;
	    
	 exception
	    when others =>
	       Text_Io.Put_Line("Error Warborg server...");
	       Sockets.Create_Selector(Selector);
	       Sockets.Set(Write,Srv_Socket);
	 end;
      end loop;
      
   end Warborg_server;


begin
   Initialize;

end Warborg.Uniborg;
