-- Warborg is Wargames playing on network
-- Copyright (C) 2011 Manuel De Girardi

-- This Program is Free Software: You Can Redistribute It and/or Modify
-- It Under The Terms of The GNU General Public License As Published By
-- The Free Software Foundation, Either Version 3 of The License, or
-- (at Your Option) Any Later Version.

-- This Program is Distributed in The Hope That It Will Be Useful,
-- But WITHOUT ANY WARRANTY; Without Even The Implied Warranty of
-- MERCHANTABILITY or FITNESS for A PARTICULAR PURPOSE.  See The
-- GNU General Public License for More Details.

-- You Should Have Received A Copy of The GNU General Public License
-- Along with This Program.  if not, See <Http://Www.Gnu.Org/Licenses/>.

with Calendar;
use Calendar;
with Ada.Containers.Vectors;


package Warborg.Common is

   Max_X_Coordonate : Float := 800.0;
   Max_y_Coordonate : Float := 600.0;

   
   subtype X_Coordonate_Type is Float range -1.0..Max_X_Coordonate+1.0;
   subtype Y_Coordonate_Type is Float range -1.0..Max_Y_Coordonate+1.0;

   type Coordonates_Type is
      record
         X : X_Coordonate_Type := 0.0;
         Y : Y_Coordonate_Type := 0.0;
      end record;
   Default_Coordonates : constant Coordonates_Type := (0.0, 0.0);

   subtype Charge_Type is Float range 0.0..1.0;

   type Missile_Attributes_Type is
      record
         Armed  : Boolean := False;
         Is_Set : Boolean := False;
         Source : Coordonates_Type := Default_Coordonates;
         Target : Coordonates_Type := Default_Coordonates;
         Charge : Charge_type      := Charge_Type'First;
      end record;
   
   type Missile_Type is
      record
         Missile_Attributes : Missile_Attributes_Type;
      end record;      
   
   
   type Side_Type is (Est, Ouest);
   
   
   type Wopr_Attributes_Type is
      record
         Source : Coordonates_Type := Default_Coordonates;
         Charge : Charge_type      := Charge_Type'Last;
         Destroyed_Wopr  : Natural := 0;
         Earned_Missiles : Natural := 0;
         Start_Date      : Time := Clock;
	 
	 Attack_Missiles   : Natural := 0;
	 Enemy_Missiles     : Natural := 0;
	 Defense_Missiles    : Natural := 0;
	 Destroyed_Missiles : Natural := 0;
	 Side               : Side_Type;
      end record;

   Index_Max        : positive := 50_000;
   type Missile_Vector_Index_Type is new Natural range 1..Index_Max;

   package Missiles_Vectors is
      new Ada.Containers.Vectors(Missile_Vector_Index_Type, Missile_type, "=");

   use Missiles_Vectors;


   type Wopr_Type is
      record
         Wopr_Attributes : Wopr_Attributes_Type;
         Missiles        : Vector;
      end record;


   type Command_Type is
     (Wopr_Create,
      Wopr_Get,
      Missile_Create,
      Launch_One,
      Launch_All,
      Enemy_get);

end Warborg.Common;
