-- Warborg is Wargames playing on network
-- Copyright (C) 2011 Manuel De Girardi

-- This Program is Free Software: You Can Redistribute It and/or Modify
-- It Under The Terms of The GNU General Public License As Published By
-- The Free Software Foundation, Either Version 3 of The License, or
-- (at Your Option) Any Later Version.

-- This Program is Distributed in The Hope That It Will Be Useful,
-- But WITHOUT ANY WARRANTY; Without Even The Implied Warranty of
-- MERCHANTABILITY or FITNESS for A PARTICULAR PURPOSE.  See The
-- GNU General Public License for More Details.

-- You Should Have Received A Copy of The GNU General Public License
-- Along with This Program.  if not, See <Http://Www.Gnu.Org/Licenses/>.

with Warborg.Common;                     use Warborg.Common;
with Warborg.Xborg;
use Warborg, Warborg.Xborg;
with Gtk.Main;

with Ada.Command_Line, Ada.Text_Io;
use Ada;
with Gnat.Os_Lib, Gnat.Sockets;
use Gnat, Gnat.Sockets;

with Reborg_Status;
with Interfaces.C;
use Interfaces;

with Ada.Unchecked_Deallocation;

procedure Xborg is


   procedure Usage is

   begin
      Text_Io.Put_Line("usage : " & Command_Line.Command_Name & " Hostname [ Logname password ]");
   end Usage;

  Args_Error : exception;
  
  Xborg             : Main_Application;
  
  function System(Command : in String) return Integer;
  pragma Import (C, System, "system");
  
  function Clear return boolean is
     Errno : Integer := 0;
   begin
      Errno := System("clear" & Character'Val(0));
      return Errno = 0;
   end Clear;
   

   procedure Print(Text : in String; Rate : in Duration := 0.05; Quiet : in Boolean := true) is
   begin
      if Text'Length > 0 then
	 for I in Text'Range loop
	    Text_Io.Put(Text(I));
	    if not Quiet and Text(I) = Character'Val(32) then
	       --Text_Io.Put(Character'Val(7));
	       null;
	    end if;
	    delay Rate;
	 end loop;
      end if;
   end Print;
   

   task type Borg_Type(Channel : Stream_Access; Logname : String_Access; Status : boolean) is	 
      entry Receive(Char : in Character);
      entry End_Of(Is_End : out Boolean);
   end Borg_Type;

   procedure Free is new Ada.Unchecked_Deallocation(String, String_Access);
   
   function To_Access(Text : in String) return String_Access is
      New_Access : String_Access;
   begin
      New_Access := new String ' (Text);
      return New_Access;
   end To_Access;
   
   task body Borg_Type is
      
      End_Of_Task : Boolean := False;
      
      Clear_Success  : Boolean := False;
      
      Logged         : Boolean := False;            
      output         : String_Access;      
      Input, buffer  : String_Access := To_Access("");      
      Quiet          : Boolean := False;
      Rate           : duration := 0.025;
      Init           : Boolean := True;
      Side           : Side_Type;
   begin      
      
      Text_Io.Put("Attend une chaine");
      Output := To_Access(String'Input(Channel));      
      
      
      loop	 	 	 	 
	 
	 if Status or Logged then
	    
	    Text_Io.Put("Attend side");
	    Side := Side_Type'Input(Channel);	    
	    Gtk.Main.Init;	    
	    Initialize(Xborg, channel, Logname, side);
	    Gtk.Main.Main;
	    End_Of_Task := True;
	    exit;
	 end if;
	 Clear_Success := Clear;
	 if not Clear_Success then
	    Text_Io.New_Line;
	 end if;	    
	 
	 if Output /= null and then Output'Length /= 0 then
	    Print(Output.all, rate, Quiet);	       
	 end if;
	 if Input /= null and then Input'Length /= 0 then
	    Print(input.all, 0.0, true);	       
	 end if;
	 delay 0.025;
	 if Init then
	    Init := False;
	    Rate := 0.0;	       
	    Quiet := True;
	 end if;

	 
	 
	 select
	    	    
	    accept Receive(Char : in Character) do
	       case Char is
		  when Character'Val(10) =>
		     if Input /= null and then Input'Length /= 0 then			
			String'Output(Channel, Input.all);
			
			Logged := Boolean'Input(Channel);
			Free(Output);
			Output := new String ' (To_Access(String'Input(Channel)).all);
			Init := not Logged;
			if Init then
			   Rate := 0.025;
			   Quiet := False;
			end if;
			Free(Input);
			Input := new String ' (To_Access("").all);
		     end if;		     
		  when Character'Val(9) =>
		     null;
		  when Character'Val(127) =>		     
		     Buffer := new String ' (To_Access(Input(Input'First..Input'Last-1)).all);
		     Input := Buffer;
		  when others =>
		     Buffer := new String ' (To_Access(Input.all & Char).all);
		     Input := Buffer;
	       end case;
	    end Receive;
	 or
	    accept End_Of(Is_End : out Boolean) do
	       Is_End := False;
	    end End_Of;
	    
	    
	 	    
	 end select;	 
      end loop;
      accept End_Of(Is_End : out Boolean) do
	 Is_End := End_Of_task;
      end End_Of;
	    
	    
   end Borg_Type;
   


  Term_Address      : Sock_Addr_Type;
  Socket            : Socket_Type;
  Channel           : Stream_access;
  Command, Response : String_Access;
  Status            : Boolean := False;
  Logname           : String_access;
  Reborg_Parameters : Os_Lib.Argument_List_Access;
  Reborg_Id         : Os_Lib.Process_Id;
  
  Success : Boolean :=False;

begin
   if Command_Line.Argument_Count < 1 then
      
      raise Args_Error;
   end if;

   Term_Address.Addr := Addresses (Get_Host_By_Name (Command_Line.Argument(1)));
   Term_Address.Port := Xborg_Port;
   Create_Socket (Socket);

   Set_Socket_Option
     (Socket,
      Socket_Level,
      (Reuse_Address, True));

   delay 0.2;

   Connect_Socket (Socket, Term_Address);

   Channel :=  stream(Socket);
   
   
   
   if Command_Line.Argument_Count = 3 then
      
      Response := new String ' (String'Input(Channel));
      Print(Response.all, 0.02, False);
      String'Output(Channel, Command_Line.Argument(2));
      Status := Boolean'Input(Channel);
      Response := new String ' (String'Input(Channel));
      Print(Response.all, 0.02, False);
      String'Output(Channel, Command_Line.Argument(3));
      Status := Boolean'Input(Channel);      
      if not Status then
         Command_Line.Set_Exit_Status(Command_Line.Failure);
         return;
      end if;
      Logname := new String ' (String'Input(Channel));
   else
      
      while not Status loop
         Response := new String ' (String'Input(Channel));
         Print(Response.all, 0.02, False);
         Command := new String ' (Text_Io.Get_Line);
         String'Output(Channel, Command.all);
         Status := Boolean'Input(Channel);
      end loop;
      Logname := new String ' (String'Input(Channel));
   end if;     
   Text_Io.Put("Attend wopr_exist");
   Status := Boolean'Input(Channel);
   declare
      
      Borg : Borg_Type(Channel, Logname, status);
   begin
      
      while not Status loop
	 
	 declare
	    Char : Character;
	    begin
	       
	       Text_Io.Get_Immediate(Char);	    
	       Borg.Receive(Char);
	       Borg.End_Of(Status);
	    exception
	       when Text_Io.End_Error =>
		  
		  raise;	    
	    end;
	 
      end loop;
      
      Borg.End_Of(Status);
   end ;
   
   
      
   
   Text_Io.Put("bye!");
   
   Close_Socket(Socket);
   
   
   if Xborg.Success then
      if Reborg_Status.End_Of_Program then
         
	 
         Reborg_Parameters := new Os_Lib.Argument_List ' (1 => new String '("Hello"),
							 2 => new String ' ("Joshua"));
         Os_Lib.Normalize_Arguments(Reborg_Parameters.all);
         Reborg_Id := Os_Lib.Non_Blocking_Spawn(Program_Name => Os_Lib.Normalize_Pathname(Reborg_Location),
						Args => Reborg_Parameters.all);	 
	 Reborg_Status.End_Of_Program := False;
      end if;
   end if;
exception

   when Args_Error =>
      Usage;
end Xborg;
